#!/bin/bash
set -e

start_time=$(date +%s)
set_color(){
	echo -e "\n\033[$1######################################\n$2 \
		         \n#############################################\033[0m\n"
}

set_color 32m \
	"【部署须知】
1、本自动化安装脚本仅适用于Kunpeng平台;
2、本脚本部署 CP2K 7.1版本
3、本脚本部署安装使用 gnu9.3 和 openmpi4.0.3
4、中途退出，请按Ctrl+C结束
"


#环境变量设置
export BISHENG=$(cd $(dirname $(which clang))/../;pwd)
export PATH=${BISHENG}/bin:$PATH
export LD_LIBRARY_PATH=${BISHENG}/lib:$LD_LIBRARY_PATH
export INCLUDE=${BISHENG}/include:$INCLUDE
export LIBEXEC=${BISHENG}/libexec:$LIBEXEC
export CC=`which mpicc`
export CXX=`which mpic++`
export FC=`which mpifort`

#设置安装包路径
export INSTALL_DIR=${PWD}
export PACKAGE_DIR=${INSTALL_DIR}
export CMAKE_DIR=${INSTALL_DIR}/CMAKE
export GMP_DIR=${INSTALL_DIR}/GMP
export BOOST_DIR=${INSTALL_DIR}/BOOST
export LIBINT_DIR=${INSTALL_DIR}/LIBINT
export FFTW_DIR=${INSTALL_DIR}/FFTW
export LAPACK_DIR=${INSTALL_DIR}/LAPACK
export SCALAPACK_DIR=${INSTALL_DIR}/SCALAPACK
export ELPA_DIR=${INSTALL_DIR}/ELPA
export SPGLIB_DIR=${INSTALL_DIR}/SPGLIB
export LIBXC_DIR=${INSTALL_DIR}/LIBXC
export GSL_DIR=${INSTALL_DIR}/GSL
export PLUMED_DIR=${INSTALL_DIR}/PLUMED
export EXTRA_DIR=${INSTALL_DIR}/EXTRA
export MATHLIB_DIR=${EXTRA_DIR}/mathlib
export CP2K_DIR=${INSTALL_DIR}/CP2K
export ENV_DIR=${INSTALL_DIR}/ENV
export LOGS_DIR=${INSTALL_DIR}/LOGS
env_check(){
  # check Arch
    if [ x$(arch) = xaarch64 ];then
            set_color 32m "Arch check ok"
    else
            set_color 31m "该脚本仅支持Kunpeng平台，请检查后再执行。。。"
    fi

    # check install dir
    if [ -d ${INSTALL_DIR} ];then
            echo "本次软件安装目录为：${INSTALL_DIR}"
    else
            echo"设置的软件目录不存在，请检查"
            exit 1
    fi


 if [ -d ${CP2K_DIR} ];then
   set_color 31m "This directory(${CP2K_DIR}) already exists"
   read -p "Please choice backup or delete ${CP2K_DIR}:(b|d)" choice
 case $choice in
 b)
   set_color 32m "You have selected file backup"
   mv ${CP2K_DIR} ${CP2K_DIR}-bak && mkdir -p ${CP2K_DIR}
   set_color 32m "Backup file done"
 ;;
 d)
    set_color 31m "You have selected file delete"
    rm -rf ${CP2K_DIR} && mkdir -p ${CP2K_DIR}
    set_color 31m "Delete file done"
  ;;
 *)
    set_color 31m "Please input 'b' or 'd'"
    exit 1
  ;;
  esac
  else
    mkdir -p ${CP2K_DIR}
    set_color 32m "${CP2K_DIR}创建成功"
  fi
}


check_install_result(){
    app=$1
    ls ${INSTALL_DIR}/${app} | wc -l
    if [ $? -eq 1 ];then
       set_color 32m "$app 检查正常，请继续..."
    else
       set_color 31m "$app 安装异常，安装中止..."
       exit 1
    fi
}

#########################################################################
#           check_packages(安装包的版本必须一致并在同一路径下)          #
#########################################################################
DBCSR_package=dbcsr-2.0.1.tar.gz
CP2K_package=cp2k-7.1.0.tar.gz

########################################################################
#	                       use_package																	 #
########################################################################
dbcsr_dir_name=dbcsr-2.0.1
cp2k_dir_name=cp2k-7.1.0


#######################################################################
#                       CP2K_7.1.0_package安装                        #
#######################################################################
install_cp2k(){
    tar -xf ${PACKAGE_DIR}/${CP2K_package} -C ${CP2K_DIR}
    cd ${CP2K_DIR}/${cp2k_dir_name}


> arch/Linux-x86-64-gfortran.psmp
			cat >> arch/Linux-x86-64-gfortran.psmp <<EOF
####################################################################################
CC           = mpicc
FC           = mpif90
LD           = mpif90
AR           = ar -r
GNU_PATH     =   \${EXTRA_DIR}
MATHLIBPATH  =   \${MATHLIB_DIR}
include      \$(GNU_PATH)/plumed252/lib/plumed/src/lib/Plumed.inc.static
ELPA_INC     =  \$(GNU_PATH)/elpa/include/elpa_openmp-2019.05.001
ELPA_LIB     =  \$(GNU_PATH)/elpa/lib
FFTW_INC     =  \$(GNU_PATH)/fftw3/include
FFTW_LIB     =  \$(GNU_PATH)/fftw3/lib
LIBINT_INC   =  \$(GNU_PATH)/libint2/include
LIBINT_LIB   =  \$(GNU_PATH)/libint2/lib
LIBXC_INC    =  \$(GNU_PATH)/libxc434/include
LIBXC_LIB    =  \$(GNU_PATH)/libxc434/lib
SPGLIB_INC   =  \$(GNU_PATH)/spglib112/include
SPGLIB_LIB   =  \$(GNU_PATH)/spglib112/lib
PLUMED_LIB   =  \$(GNU_PATH)/plumed252/lib
PLUMED_INC   =  \$(GNU_PATH)/plumed252/include
GSLBLAS_LIB  =  \$(GNU_PATH)/gsl/lib
GSLBLAS_INC  =  \$(GNU_PATH)/gsl/include
CFLAGS       = -O1 -g -mtune=native -fno-unroll-loops -fstack-protector
DFLAGS       = -D__ELPA -D__FFTW3 -D__LIBINT -D__LIBXC
DFLAGS       += -D__MPI_VERSION=3 -D__PLUMED2 -D__SPGLIB
DFLAGS       += -D__parallel -D__SCALAPACK
FCFLAGS      = \$(CFLAGS) \$(DFLAGS)
FCFLAGS      += -ffree-form -ffree-line-length-none
FCFLAGS      += -fopenmp -fno-omit-frame-pointer
FCFLAGS      += -ftree-vectorize -funroll-loops -std=f2008
FCFLAGS      += -I\$(ELPA_INC)/elpa -I\$(ELPA_INC)/modules
FCFLAGS      += -I\$(FFTW_INC) -I\$(LIBINT_INC) -I\$(LIBXC_INC) -I\$(GSLBLAS_INC) -I\$(PLUMED_INC)
LDFLAGS      = \$(FCFLAGS) -static-flang-libs
LIBS         = -L\$(GSLBLAS_LIB) -L\$(PLUMED_LIB) -lgsl -lgslcblas -lz -lplumed -lplumedKernel
LIBS         += \$(ELPA_LIB)/libelpa_openmp.a
LIBS         += \$(LIBXC_LIB)/libxcf03.a
LIBS         += \$(LIBXC_LIB)/libxc.a
LIBS         += \$(LIBINT_LIB)/libint2.a
LIBS         += \$(SPGLIB_LIB)/libsymspg.a
LIBS         += \$(FFTW_LIB)/libfftw3.a
LIBS         += \$(FFTW_LIB)/libfftw3_threads.a
LIBS         += \$(MATHLIBPATH)/libscalapack.a
LIBS         += \$(MATHLIBPATH)/liblapack.a
LIBS         += \$(MATHLIBPATH)/libblas.a
LIBS         += -ldl -lpthread -lstdc++
####################################################################################
EOF

    ###设置CP2K-7.1.0安装所需的环境变量###
    export LIBRARY_PATH=${EXTRA_DIR}/gsl/lib:\$LIBRARY_PATH
    export LD_LIBRARY_PATH=${EXTRA_DIR}/gsl/lib:\$LD_LIBRARY_PATH
    export LD_LIBRARY_PATH=${PLUMED_DIR}/lib:\$LD_LIBRARY_PATH


    cd ${CP2K_DIR}/${cp2k_dir_name}/exts/dbcsr
    tar -xf ${PACKAGE_DIR}/${DBCSR_package} -C ${CP2K_DIR}/${cp2k_dir_name}/exts/dbcsr
    cd ${CP2K_DIR}/${cp2k_dir_name}/exts/dbcsr/${dbcsr_dir_name}
    mv * ${CP2K_DIR}/${cp2k_dir_name}/exts/dbcsr
    cd ${CP2K_DIR}/${cp2k_dir_name}
    make -j16 ARCH=Linux-x86-64-gfortran VERSION=psmp

    ###设置CP2K-7.1.0测试所需执行文件的环境变量###
    export PATH=${CP2K_DIR}/cp2k-7.1.0/exe/Linux-x86-64-gfortran:$PATH
}

#######################################################################
#                 如需测试CP2K-7.1.0，则需要加载环境变量              #
#######################################################################
print_env(){
  set_color 31m "如需执行CP2K-7.1.0，则需要先加载环境变量: source ./ENV/CP2K_env.sh"
  echo "export PATH=${CP2K_DIR}/${cp2k_dir_name}/exe/Linux-x86-64-gfortran:\$PATH"
  echo "export LD_LIBRARY_PATH=${EXTRA_DIR}/gsl/lib:\$LD_LIBRARY_PATH"
}

[ -f ${PWD}/ENV/CP2K_env.sh ] && rm -rf ${PWD}/ENV/CP2K_env.sh
echo "export LD_LIBRARY_PATH=${EXTRA_DIR}/gsl/lib:\$LD_LIBRARY_PATH" > ${PWD}/ENV/CP2K_env.sh
echo "export PATH=${CP2K_DIR}/${cp2k_dir_name}/exe/Linux-x86-64-gfortran:\$PATH" >> ${PWD}/ENV/CP2K_env.sh

time_tag=$(echo $(date +%F-%H%M%S)_$(hostname))
env_check
install_cp2k 2>&1 | tee -a  $LOGS_DIR/install_CP2K_aarch64_gcc_${time_tag}.log
end_time=$(date +%s)
spend_time=$( expr ${end_time} - ${start_time} )
set_color 33m "CP2K安装成功，本次安装耗时：${spend_time}s"
