#!/bin/bash
set -e

WORK_HOME=`pwd`
INS_HOME_PATH="${WORK_HOME}/application"

max_jobs=$(nproc)
if [[ ${max_jobs} -gt 32 ]]; then
    max_jobs=32
fi

#设置环境
export FFTW_LIB=${WORK_HOME}/FFTW/lib
export FFTW_INC=${WORK_HOME}/FFTW/include
export MATH_LIB=${WORK_HOME}/MATH/lib
export CMAKE_DIR=${WORK_HOME}/CMAKE/bin
export BLAS_LIB=${WORK_HOME}/OpenBLAS/lib



function gromacs_install()
{
    cd ${WORK_HOME}/gromacs-2019.3
    mkdir build && cd build
    FLAGS="-mcpu=tsv110"; CFLAGS=$FLAGS CXXFLAGS=$FLAGS LDFLAGS="-lgfortran" CC=mpicc CXX=mpicxx \
    ${CMAKE_DIR}/cmake -DCMAKE_INSTALL_PREFIX=${INS_HOME_PATH} \
    -DBUILD_SHARED_LIBS=on \
    -DBUILD_TESTING=on \
    -DREGRESSIONTEST_DOWNLOAD=off \
    -DGMX_BUILD_OWN_FFTW=off \
    -DGMX_SIMD=ARM_NEON_ASIMD \
    -DGMX_DOUBLE=off \
    -DGMX_EXTERNAL_BLAS=on \
    -DGMX_EXTERNAL_LAPACK=on \
    -DGMX_BLAS_USER=${BLAS_LIB}/libopenblas.a \
    -DGMX_LAPACK_USER=${BLAS_LIB}/libopenblas.a \
    -DFFTWF_LIBRARY=${FFTW_LIB}/libfftw3f.so \
    -DFFTWF_INCLUDE_DIR=${FFTW_INC} \
    -DGMX_GPU=off \
    -DGMX_MPI=on \
    -DGMX_OPENMP=on \
    -DGMX_X11=off \
    ../ 2>&1 | tee gromacs_cmake.log
    make -j ${max_jobs} V=1
    make -j ${max_jobs} install  2>&1 | tee gromacs_install.log
    cp gromacs_cmake.log gromacs_install.log ${INS_HOME_PATH}
}

gromacs_install
