#!/bin/bash
set -e
export MPI_HOME=$(dirname $(dirname $(which mpirun)))

HOST=$PWD
INS_HOME_PATH="${HOST}/application"
SRC_HOME_PATH="${HOST}/source"

HDF5=${HOST}/HDF5
PNETCDF=${HOST}/PNETCDF
NETCDF=${HOST}/NETCDF
MATH=${HOST}/MATH
BLITZ=${HOST}/BLITZ
XIOS=${HOST}/XIOS_

max_jobs=$(nproc)
if [[ ${max_jobs} -gt 32 ]]; then
    max_jobs=32
fi

function hdf5_install()
{
    cd ${SRC_HOME_PATH}/hdf5-1.12.1
    ./configure --prefix=${HDF5} --build=aarch64-unknown-linux-gnu --enable-fortran --enable-static=yes --enable-parallel --enable-shared CC=mpicc CXX=mpicxx FC=mpifort F77=mpifort
    make -j ${max_jobs} && make install
}


function pnetcdf_install()
{
   cd ${SRC_HOME_PATH}/pnetcdf-1.12.2
   ./configure --prefix=${PNETCDF} --build=aarch64-unknown-linux-gnu  CFLAGS="-fPIC -DPIC" CXXFLAGS="-fPIC -DPIC" FCFLAGS="-fPIC" FFLAGS="-fPIC" CC=mpicc CXX=mpicxx FC=mpifort F77=mpifort
   make -j ${max_jobs} && make install
}


function netcdf_install()
{
    cd ${SRC_HOME_PATH}/netcdf-c-4.4.1.1
    ./configure --prefix=${NETCDF} --build=aarch64-unknown-linux-gnu --enable-shared --enable-netcdf-4 --enable-dap --with-pic --disable-doxygen --enable-static --enable-pnetcdf --enable-largefile CC=mpicc CXX=mpicxx FC=mpifort F77=mpifort CPPFLAGS="-I$HDF5/include -I$PNETCDF/include" LDFLAGS="-L$HDF5/lib -L$PNETCDF/lib -Wl,-rpath=$HDF5/lib -Wl,-rpath=$PNETCDF/lib" CFLAGS="-L$HDF5/lib -L$PNETCDF/lib -I$HDF5/include -I$PNETCDF/include"
    make -j ${max_jobs} && make install
}


function netcdff_install()
{
    cd ${SRC_HOME_PATH}/netcdf-fortran-4.4.1
    ./configure --prefix=$NETCDF --build=aarch64-unknown-linux-gnu --enable-shared --with-pic --disable-doxygen --enable-largefile --enable-static CC=mpicc CXX=mpicxx FC=mpifort F77=mpifort CPPFLAGS="-I$HDF5/include -I$NETCDF/include" LDFLAGS="-L$HDF5/lib -L$NETCDF/lib -Wl,-rpath=$HDF5/lib -Wl,-rpath=$NETCDF/lib" CFLAGS="-L$HDF5/lib -L$NETCDF/lib -I$HDF5/include -I$NETCDF/include" CXXFLAGS="-L$HDF5/lib -L$NETCDF/lib -I$HDF5/include -I$NETCDF/include" FCFLAGS="-L$HDF5/lib -L$NETCDF/lib -I$HDF5/include -I$NETCDF/include"
    make -j ${max_jobs} && make install
}


function MATH_install()
{
    cd ${SRC_HOME_PATH}/optimized-routines-201910
    cp config.mk.dist config.mk
    make -j ${max_jobs}
    cp -r ./build/* $MATH
}


function blitz_install()
{
    cd ${SRC_HOME_PATH}/blitz-1.0.1
    autoreconf -fiv
    ./configure --prefix=${BLITZ} --enable-fortran --enable-64bit
    make lib && make install 
    cp -rf ./src ${BLITZ}
}


function xios_install()
{
    cd ${SRC_HOME_PATH}/XIOS/branchs/xios-1.0
    sed -ri "s|^src::date|#src::date/|g" bld.cfg
    sed -ri "s/^bld::target xios_server.exe/#bld::target xios_server.exe/" bld.cfg
    
    cat > ${SRC_HOME_PATH}/XIOS/branchs/xios-1.0/arch/arch-AARCH64_LINUX.env << EOF
export HDF5_INC_DIR=${HDF5}/include
export HDF5_LIB_DIR=${HDF5}/lib

export NETCDF_INC_DIR=${NETCDF}/include
export NETCDF_LIB_DIR=${NETCDF}/lib

export BOOST_INC_DIR="/usr/include/boost/"
export BOOST_LIB_DIR="/usr/lib64/"

export BLITZ_INC_DIR=${BLITZ}/include
export BLITZ_LIB_DIR=${BLITZ}/lib
EOF
    cat > ${SRC_HOME_PATH}/XIOS/branchs/xios-1.0/arch/arch-AARCH64_LINUX.fcm << EOF
################################################################################
###################        Projet xios - xmlioserver       #####################
################################################################################

%CCOMPILER      mpicc
%FCOMPILER      mpif90
%LINKER         mpif90

%BASE_CFLAGS    -ansi -w -D_GLIBCXX_USE_CXX11_ABI=0
%PROD_CFLAGS    -O3 -DBOOST_DISABLE_ASSERTS
%DEV_CFLAGS     -g -O2
%DEBUG_CFLAGS   -g

%BASE_FFLAGS    -D__NONE__
%PROD_FFLAGS    -O3
%DEV_FFLAGS     -g -O2
%DEBUG_FFLAGS   -g

%BASE_INC       -D__NONE__
%BASE_LD        -lstdc++

%CPP            cpp
%FPP            cpp -P
%MAKE           gmake
EOF
    cat > ${SRC_HOME_PATH}/XIOS/branchs/xios-1.0/arch/arch-AARCH64_LINUX.path << EOF
NETCDF_INCDIR="-I ${NETCDF}/include"
NETCDF_LIBDIR="-L ${NETCDF}/lib"
NETCDF_LIB="-lnetcdff -lnetcdf"

MPI_INCDIR="-I ${MPI_HOME}/include"
MPI_LIBDIR="-L ${MPI_HOME}/lib"
MPI_LIB="-lmpi"

HDF5_INCDIR="-I ${HDF5}/include"
HDF5_LIBDIR="-L ${HDF5}/lib"
HDF5_LIB="-lhdf5_hl -lhdf5  -lz"

BOOST_INCDIR="-I /usr/include/boost/"
BOOST_LIBDIR="-L /usr/lib64/"
BOOST_LIB=""

BLITZ_INCDIR="-I ${BLITZ}/include"
BLITZ_LIBDIR="-L ${BLITZ}/lib"
BLITZ_LIB="-lblitz"
EOF
    chmod +x ./make_xios
    cd ${SRC_HOME_PATH}/XIOS/branchs/xios-1.0/extern
    ln -s ${BLITZ} ./blitz
    ln -s ${NETCDF} ./netcdf4
    ln -s /usr/include/boost ./boost
    cd ../
    { ./make_xios --dev --job ${max_jobs} --full --arch AARCH64_LINUX ; } 2>&1 |tee install-1.log
    cd ${SRC_HOME_PATH}/XIOS/branchs/xios-1.0/extern
    rm -rf ./blitz
    rm -rf ./netcdf4
    rm -rf ./boost
    ln -s ${BLITZ} ./blitz
    ln -s ${NETCDF} ./netcdf4
    ln -s /usr/include/boost ./boost
    cd ../
    { ./make_xios --dev --job ${max_jobs} --full --arch AARCH64_LINUX ; } 2>&1 |tee install-2.log
    mkdir -p ${XIOS}/lib ${XIOS}/include
    cp ${SRC_HOME_PATH}/XIOS/branchs/xios-1.0/lib/libxios.a ${XIOS}/lib
    cp ${SRC_HOME_PATH}/XIOS/branchs/xios-1.0/inc/* ${XIOS}/include
}

arch=$(arch)
case $arch in 
    aarch64)
        echo ""
        echo "The hardware platform is aarch64"
        echo 
        ;;
    *)
        echo "Please check you Architecture!"
        exit 1
        ;;
esac

hdf5_install
pnetcdf_install
netcdf_install
netcdff_install
MATH_install
blitz_install
xios_install
