#!/bin/bash
#module use /Oceanfile/RR3-compiler-env/module_file/aarch64/
#module add gnu-9.3.0 omp4.0.3-gcc9.3.0
set -e
export MPI_HOME=$(dirname $(dirname $(which mpirun)))

export BISHENG=$(cd $(dirname $(which clang))/../;pwd)
export PATH=${BISHENG}/bin:$PATH
export LD_LIBRARY_PATH=${BISHENG}/lib:$LD_LIBRARY_PATH
export INCLUDE=${BISHENG}/include:$INCLUDE
export LIBEXEC=${BISHENG}/libexec:$LIBEXEC
export CC=clang
export CXX=clang++
export FC=flang
export MPICC=mpicc
export MPICXX=mpicxx
export MPIFC=mpifort

HOST=$PWD
INS_HOME_PATH="${HOST}/application"
SRC_HOME_PATH="${HOST}/source"

HDF5=${HOST}/HDF5
PNETCDF=${HOST}/PNETCDF
NETCDF=${HOST}/NETCDF
BLITZ=${HOST}/BLITZ
XIOS=${HOST}/XIOS_
ZLIB=${HOST}/ZLIB


max_jobs=$(nproc)
if [[ ${max_jobs} -gt 32 ]]; then
    max_jobs=32
fi

function hdf5_install()
{
   cd ${SRC_HOME_PATH}/hdf5-1.12.1
   CC=mpicc CXX=mpicxx FC=mpifort F77=mpi77 CFLAGS="-fPIC -DPIC" FCFLAGS="-fPIC -DPIC" FFLAGS="-fPIC" \
   ./configure --prefix=${HDF5} --with-zlib=${ZLIB} --enable-shared --enable-fortran --enable-static --enable-parallel
   sed -i 's@\\\$wl-soname@-install_name@g' libtool
   make -j $(nproc) && make -j $(nproc) install
   export PATH=${HDF5}bin:$PATH
   export LD_LIBRARY_PATH=${HDF5}/lib:$LD_LIBRARY_PATH
}

function pnetcdf_install()
{
   cd ${SRC_HOME_PATH}/pnetcdf-1.12.2
  CC=mpicc CXX=mpicxx FC=mpifort F77=mpifort CFLAGS="-fPIC -DPIC" FCFLAGS="-fPIC" FFLAGS="-fPIC" CXXFLAGS="-fPIC -DPIC" SEQ_CC=mpicc \
  ./configure --prefix=${PNETCDF} --enable-shared
  make -j$(nproc) && make -j$(nproc) install
  export PATH=${PNETCDF}/bin:$PATH
  export LD_LIBRARY_PATH=${PNETCDF}/lib:$LD_LIBRARY_PATH
}

function netcdf_c_install()
{
    cd ${SRC_HOME_PATH}/netcdf-c-4.8.1
    HDF5_LIB=${HDF5}/lib ZLIB_LIB=/usr/lib64 PNETCDF_LIB=${PNETCDF}/lib \
    HDF5_INCLUDE=${HDF5}/include ZLIB_INCLUDE=/usr/include PNETCDF_INCLUDE=${PNETCDF}/include \
    CC=mpicc CXX=mpicxx CPPFLAGS="-I${HDF5_INCLUDE} -I${ZLIB_INCLUDE} -I${PNETCDF_INCLUDE}" \
    CFLAGS="-I${HDF5_INCLUDE}  -I${ZLIB_INCLUDE} -I${PNETCDF_INCLUDE} -fPIC" \
    LDFLAGS="-L${HDF5_LIB}  -L${ZLIB_LIB} -L${PNETCDF_LIB}" \
    ./configure --prefix=${NETCDF} --enable-shared --enable-netcdf-4 --enable-dap --with-pic --disable-doxygen --enable-pnetcdf --enable-static --enable-largefile
    make -j$(nproc) && make -j$(nproc) install

    export PATH=${NETCDF}/bin:$PATH
    export LD_LIBRARY_PATH=${NETCDF}/lib:$LD_LIBRARY_PATH
    }

function netcdf_fortran_install()
{
    export LD_LIBRARY_PATH=${NETCDF}/lib:$LD_LIBRARY_PATH
    cd ${SRC_HOME_PATH}/netcdf-fortran-4.5.3
    CC=mpicc CXX=mpicxx FC=mpifort F77=mpifort \
    HDF5_LIB=${HDF5}/lib NETCDF_LIB=${NETCDF}/lib \
    HDF5_INCLUDE=${HDF5}/include NETCDF_INCLUDE=${NETCDF}/include \
    CPPFLAGS="-I${HDF5_INCLUDE} -I${NETCDF_INCLUDE}" \
    CFLAGS="-I${HDF5_INCLUDE} -I${NETCDF_INCLUDE} -fPIC -DPIC" \
    FCFLAGS="-I${HDF5_INCLUDE} -I${NETCDF_INCLUDE} -fPIC -DPIC" \
    LDFLAGS="-L${HDF5_LIB} -L${NETCDF_LIB}" \
    ./configure --prefix=${NETCDF} --enable-shared --enable-dap --enable-pic --disable-doxygen --enable-static --enable-largefile
    sed -i \
      -e 's/\\\$wl-soname/-install_name/g' \
      -e 's/\\\$wl--whole-archive/-Wl,-force-load,/g' \
      -e 's/\\\$wl--no-whole-archive//g' libtool
    make -j$(nproc) && make -j$(nproc) install

    export PATH=${NETCDF}/bin:$PATH
    export LD_LIBRARY_PATH=${NETCDF}/lib:$LD_LIBRARY_PATH
}

function blitz_install()
{
    cd ${SRC_HOME_PATH}/blitz-1.0.1
    autoreconf -fiv
    ./configure --prefix=${BLITZ} --enable-fortran --enable-64bit
    make lib && make install 
    cp -rf ./src ${BLITZ}
    export BLITZ_INCLUDEDIR=${BLITZ}/include
    export LD_LIBRARY_PATH=${BLITZ}/lib:$LD_LIBRARY_PATH
}

function xios_install()
{
    MPI_HOME=$(cd $(dirname $(which mpicc))/../../;pwd)
    cd ${SRC_HOME_PATH}/XIOS/branchs/xios-1.0
    sed -ri "s|^src::date|#src::date/|g" bld.cfg
    sed -ri "s/^bld::target xios_server.exe/#bld::target xios_server.exe/" bld.cfg
    
    cat > ${SRC_HOME_PATH}/XIOS/branchs/xios-1.0/arch/arch-AARCH64_LINUX.env << EOF
export HDF5_INC_DIR=${HDF5}/include
export HDF5_LIB_DIR=${HDF5}/lib

export NETCDF_INC_DIR=${NETCDF}/include
export NETCDF_LIB_DIR=${NETCDF}/lib

export BOOST_INC_DIR="/usr/include/boost/"
export BOOST_LIB_DIR="/usr/lib64/"

export BLITZ_INC_DIR=${BLITZ}/include
export BLITZ_LIB_DIR=${BLITZ}/lib
EOF
    cat > ${SRC_HOME_PATH}/XIOS/branchs/xios-1.0/arch/arch-AARCH64_LINUX.fcm << EOF
################################################################################
###################        Projet xios - xmlioserver       #####################
################################################################################

%CCOMPILER      mpicc
%FCOMPILER      mpif90
%LINKER         mpif90

%BASE_CFLAGS    -ansi -w -D_GLIBCXX_USE_CXX11_ABI=0
%PROD_CFLAGS    -O3 -DBOOST_DISABLE_ASSERTS
%DEV_CFLAGS     -g -O2
%DEBUG_CFLAGS   -g

%BASE_FFLAGS    -D__NONE__
%PROD_FFLAGS    -O3
%DEV_FFLAGS     -g -O2
%DEBUG_FFLAGS   -g      #-trackback

%BASE_INC       -D__NONE__
%BASE_LD        -lstdc++

%CPP            cpp
%FPP            cpp -P
%MAKE           make
EOF
    cat > ${SRC_HOME_PATH}/XIOS/branchs/xios-1.0/arch/arch-AARCH64_LINUX.path << EOF
NETCDF_INCDIR="-I ${NETCDF}/include"
NETCDF_LIBDIR="-L ${NETCDF}/lib"
NETCDF_LIB="-lnetcdff -lnetcdf"

MPI_INCDIR="-I ${MPI_HOME}/include"
MPI_LIBDIR="-L ${MPI_HOME}/lib"
MPI_LIB="-lmpi"

HDF5_INCDIR="-I ${HDF5}/include"
HDF5_LIBDIR="-L ${HDF5}/lib"
HDF5_LIB="-lhdf5_hl -lhdf5  -lz"

BOOST_INCDIR="-I /usr/include/boost/"
BOOST_LIBDIR="-L /usr/lib64/"
BOOST_LIB=""

BLITZ_INCDIR="-I ${BLITZ}/include"
BLITZ_LIBDIR="-L ${BLITZ}/lib"
BLITZ_LIB="-lblitz"
EOF
  chmod +x ./make_xios
  cd ${SRC_HOME_PATH}/XIOS/branchs/xios-1.0/extern
  ln -s ${BLITZ} ./blitz
  ln -s ${NETCDF} ./netcdf4
  ln -s /usr/include/boost ./boost
  cd ../
  { ./make_xios --dev --job ${max_jobs} --full --arch AARCH64_LINUX ; } 2>&1 |tee install-1.log
  # 第一次编译执行后会报错，再执行一遍以下命令
  cd ${SRC_HOME_PATH}/XIOS/branchs/xios-1.0/extern
  rm -rf ./blitz
  rm -rf ./netcdf4
  rm -rf ./boost
  ln -s ${BLITZ} ./blitz
  ln -s ${NETCDF} ./netcdf4
  ln -s /usr/include/boost ./boost
  cd ../
  { ./make_xios --dev --job ${max_jobs} --full --arch AARCH64_LINUX ; } 2>&1 |tee install-2.log
  mkdir -p ${XIOS}/lib ${XIOS}/include
  cp ${SRC_HOME_PATH}/XIOS/branchs/xios-1.0/lib/libxios.a ${XIOS}/lib
  cp ${SRC_HOME_PATH}/XIOS/branchs/xios-1.0/inc/* ${XIOS}/include
}


arch=$(arch)
case $arch in 
    aarch64)
        echo ""
        echo "The hardware platform is aarch64"
        echo 
        ;;
    *)
        echo "Please check you Architecture!"
        exit 1
        ;;
esac

hdf5_install
pnetcdf_install
netcdf_c_install
netcdf_fortran_install
blitz_install
xios_install
