#!/bin/bash

### Description: This is a Frame Script, which is used for HPC Application automatic compilation and installation !!! ###
###                                         Version Number: V 1.0                                                     ###
###                                       Released Date: 2021/4/23                                                   ###
#########################################################################################################################

######################################## Don't Modify####################################################################

### Global Definition ###
HOST=$PWD
INS_HOME_PATH="${HOST}/application"
SRC_HOME_PATH="${HOST}/source"
ENV_PATH="${HOST}/env"
Architecture=`uname -m`

### Define the src path and installation path ###
#=== Build Path ===
mkdir -p ${SRC_HOME_PATH}/${APPLICATION}/${VERSION}/
mkdir -p ${INS_HOME_PATH}/${Architecture}/${APPLICATION}/${VERSION}
mkdir -p ${ENV_PATH}/${Architecture}/${APPLICATION}

export MPI_HOME=$(dirname $(dirname $(which mpirun)))

export BISHENG=$(cd $(dirname $(which clang))/../;pwd)
export PATH=${BISHENG}/bin:$PATH
export LD_LIBRARY_PATH=${BISHENG}/lib:$LD_LIBRARY_PATH
export INCLUDE=${BISHENG}/include:$INCLUDE
export LIBEXEC=${BISHENG}/libexec:$LIBEXEC

export CC=`which clang`
export CXX=`which clang++`
export FC=`which flang`

#定义编译和安装相关函数
function NWChem_decompress()
{
   #判断源码包是否下载
   if [ ! -f ${HOST}/nwchem-6.8.1-release.tar.gz ]; then
        echo "Package download failed, please check the network connection and others"
        exit 1
   else
        echo "The NWChem Source Packge is allready exiting, no need to download again"
   fi
   #解压源码包
   if [ ! -d ${INS_HOME_PATH}/nwchem-6.8.1-release ]; then
      tar -xf ${HOST}/nwchem-6.8.1-release.tar.gz -C ${INS_HOME_PATH}
      # 修改NWCHEM_TOP 最长字符限制
      sed -ri '23 s/64/1024/g' ${INS_HOME_PATH}/nwchem-6.8.1-release/src/util/GNUmakefile
      sed -i '24i \   LIB_DEFINES += -DNWCHEM_LONG_PATHS' ${INS_HOME_PATH}/nwchem-6.8.1-release/src/util/GNUmakefile
   fi
}


function  GlobalArray_decompress()
{
   #判断源码包是否下载
   if [ ! -f ${HOST}/ga-5.6.5.tar.gz ]; then
        echo "Package download failed, please check the network connection and others"
        exit 1
   else
        echo "The GlobalArray Source Packge is allready exiting, no need to download again"
   fi
   #解压源码包
   if [ ! -d ${INS_HOME_PATH}/nwchem-6.8.1-release/src/tools/ga-5.6.5 ]; then
      cp ga-5.6.5.tar.gz ${INS_HOME_PATH}/nwchem-6.8.1-release/src/tools
      tar xf ga-5.6.5.tar.gz -C ${INS_HOME_PATH}/nwchem-6.8.1-release/src/tools
   fi    
}
function nwchem_make()
{
    cd ${INS_HOME_PATH}/nwchem-6.8.1-release/src
    export NWCHEM_TOP=${INS_HOME_PATH}/nwchem-6.8.1-release
    export NWCHEM_TARGET=LINUX64
    export NWCHEM_MODULES=all
    export USE_MPI=y
    export USE_INTERNALBLAS=y
    make nwchem_config
    make FC=flang
}

### Hardware Architecture, Select Baseline Combination, Load Module File ###
case $Architecture in
aarch64)
  echo ""
  echo "The hardware platform is aarch64"
  echo ""
;;
*)
  exit
  ;;
esac


NWChem_decompress
GlobalArray_decompress
nwchem_make

