#!/bin/bash

### Description: This is a Frame Script, which is used for HPC Application automatic compilation and installation !!! ###
###                                         Version Number: V 1.0                                                     ###
###                                       Released Date: 2019/11/20                                                   ###
#########################################################################################################################

######################################## Don't Modify####################################################################

### Global Definition ###
# 1. Modify "APPLICATION" and "VERSION"
APPLICATION="OpenFOAM"
VERSION="v1906"
HOST=$PWD
HMPI=${HOST}/HMPI
INS_HOME_PATH="${HOST}/application"
SRC_HOME_PATH="${HOST}/source"
ENV_PATH="${HOST}/env"
Architecture=`uname -m`

### Define the src path and installation path ###
#=== Build Path ===
mkdir -p ${SRC_HOME_PATH}/${APPLICATION}/${VERSION}/
mkdir -p ${INS_HOME_PATH}/${Architecture}/${APPLICATION}/${VERSION}
mkdir -p ${ENV_PATH}/${Architecture}/${APPLICATION}
export BISHENG=$(cd $(dirname $(which clang))/../;pwd)
export PATH=${BISHENG}/bin:$PATH
export LD_LIBRARY_PATH=${BISHENG}/lib:$LD_LIBRARY_PATH
export INCLUDE=${BISHENG}/include:$INCLUDE
export LIBEXEC=${BISHENG}/libexec:$LIBEXEC

function openfoam_unpack()
{
   if [ ! -d ${INS_HOME_PATH}/${Architecture}/${APPLICATION}/${VERSION}/${APPLICATION}-${VERSION} ]; then
      tar -zxvf ${APPLICATION}-${VERSION}.tgz \
      -C ${INS_HOME_PATH}/${Architecture}/${APPLICATION}/${VERSION}
   fi
}

function thirdparty_unpack()
{
   if [ ! -d ${INS_HOME_PATH}/${Architecture}/${APPLICATION}/${VERSION}/ThirdParty-${VERSION} ]; then
      tar -zxvf ThirdParty-${VERSION}.tgz \
      -C ${INS_HOME_PATH}/${Architecture}/${APPLICATION}/${VERSION}
   fi
}

### Begin to unpack all packages ###
# 3. Change to your own define function name
openfoam_unpack
thirdparty_unpack

cd ${INS_HOME_PATH}/${Architecture}/${APPLICATION}/${VERSION}/${APPLICATION}-${VERSION}
sed -i "161c DynamicList<char>&& buffer," src/OpenFOAM/db/IOstreams/memory/IListStream.H
sed -i "77c export WM_COMPILER=Clang" etc/bashrc
sed -i "85c export WM_LABEL_SIZE=64" etc/bashrc
source etc/bashrc
./Allwmake -j $(nproc)
echo 'compilation success'
