#!/bin/bash

#############################QE 6.4.1 安装

start_time=$(date +%s)
set_color()
{
  echo -e "\n\033[$1######################################\n$2 \
				 \n########################################\033[0m\n"
}

set_color 32m \
  "【部署须知】
1、本自动化安装脚本仅适用于KUNPENG平台;
2、本脚本部署 QUANTUM ESPRESSO 6.4.1版本
3、本脚本部署安装使用GCC-9.3 和 omp4.0.3-gcc9.3.0
4、中途退出，请按Ctrl+C结束
"

#环境变量设置(在此处加载gcc和openmpi的环境变量，并设置安装包的路径)

export INSTALL_DIR=${PWD}
export OPENBLAS_DIR=$INSTALL_DIR/OPENBLAS
export SCALAPACK_DIR=$INSTALL_DIR/SCALAPACK
export QE_DIR=$INSTALL_DIR/QE
export PACKAGE_DIR=${PWD}
export LOGS_DIR=$INSTALL_DIR/LOGS
export ENV_DIR=$INSTALL_DIR/ENV
export CC=`which mpicc`
export CXX=`which mpic++`
export FC=`which mpifort`

check_install_result()
{
  app=$1
  ls $INSTALL/$app | wc -l
  if [ $? -eq 1 ]; then
    set_color 32m "$app 检查正常，请继续"
  else
    set_color 31m "$app 安装异常，安装中止...."
    exit 1
  fi
}

#check_package（安装包的版本必须一至并在同一路径下）
qe_package=qe-6.4.1.tar.gz


#QE的安装
install_qe()
{
  tar -zxvf $PACKAGE_DIR/$qe_package -C $QE_DIR
  export BLAS_LIBS="-L$OPENBLAS_DIR/lib -lopenblas"
  export LAPACK_LIBS="-L$OPENBLAS_DIR/lib -lopenblas"
  export SCALAPACK_LIBS="-L$SCALAPACK_DIR -lscalapack"
  cd $QE_DIR/q-e-qe-6.4.1
  ./configure F90=gfortran F77=gfortran MPIF90=mpifort MPIF77=mpifort CC=mpicc FCFLAGS="-O3" CFLAGS="-O3" --with-scalapack=yes --prefix=$QE_DIR
  make -j 20 pwall
  make install
  export PATH=$QE_DIR/bin:$PATH
}

env_use()
{
  set_color 31m "如需使用请加载相应环境变量"
  echo "export PATH=$QE_DIR/bin:\$PATH" | tee -a $ENV_DIR/QE_env_aarch64_gcc.sh
  echo "export LD_LIBRARY_PATH=$OPENBLAS_DIR/lib:\$LD_LIBRARY_PATH" | tee -a $ENV_DIR/QE_env_aarch64_gcc.sh
}


time_tag=$(echo $(date +%F-%H%M%S)_$(hostname))
install_qe 2>&1 | tee -a $LOGS_DIR/install_QE_aarch64_gcc_${time_tag}.log
env_use
end_time=$(date +%s)
spend_time=$(expr $end_time - $start_time)
set_color 33m "本次安装耗时：${spend_time}s"
