#!/bin/bash


#############################QE 6.4.1 安装

start_time=$(date +%s)
set_color()
{
  echo -e "\n\033[$1######################################\n$2 \
				 \n########################################\033[0m\n"
}

set_color 32m \
  "【部署须知】
1、本自动化安装脚本仅适用于KUNPENG平台;
2、本脚本部署 QUANTUM ESPRESSO 6.4.1版本
3、本脚本部署安装使用GCC-9.3 和 omp4.0.3-gcc9.3.0
4、中途退出，请按Ctrl+C结束
"

#环境变量设置(在此处加载gcc和openmpi的环境变量，并设置安装包的路径)


export INSTALL_DIR=${PWD}
export OPENBLAS_DIR=$INSTALL_DIR/OPENBLAS
export SCALAPACK_DIR=$INSTALL_DIR/SCALAPACK
export QE_DIR=$INSTALL_DIR/QE
export PACKAGE_DIR=${PWD}
export LOGS_DIR=$INSTALL_DIR/LOGS
export ENV_DIR=$INSTALL_DIR/ENV
mkdir -p $LOGS_DIR $ENV_DIR

env_check()
{
  # check Arch
  if [ x$(arch) = xaarch64 ]; then
    set_color 32m "Arch check ok"
  else
    set_color 31m "该脚本仅支持aarch64，请检查后再执行。。。"
  fi

  # check install dir
  if [ -d $INSTALL_DIR ]; then
    echo "本次软件安装目录为: $INSTALL_DIR"
  else
    echo "设置的软件目录不存在，请检查"
    exit 1
  fi

  DIRS=(
    $QE_DIR
    $OPENBLAS_DIR
    $SCALAPACK_DIR
  )
  for i in ${DIRS[@]}; do
    if [ -d $i ]; then
      set_color 31m "This directory($i) already exists"
      read -p "please choice backup or delete $i:(b|d)" choice
      case $choice in
        b)
          set_color 32m "You have selected file backup"
          mv $i $i.$(date +%F-%H%M%S) && mkdir -p $i
          set_color 32m "Backup file done"
          ;;
        d)
          set_color 31 "You have selected file delete"
          rm -rf $i && mkdir -p $i
          set_color 31m "Delete file done"
          ;;
        *)
          set_color 31m "Please input 'b' or 'd'"
          exit 1
          ;;
      esac
    else
      mkdir -p $i
      set_color 32m "$i创建成功"
    fi
  done
}

check_install_result()
{
  app=$1
  ls $INSTALL/$app | wc -l
  if [ $? -eq 1 ]; then
    set_color 32m "$app 检查正常，请继续"
  else
    set_color 31m "$app 安装异常，安装中止...."
    exit 1
  fi
}

#check_package（安装包的版本必须一至并在同一路径下）
scalapack_package=scalapack-2.0.2.tar.gz
OpenBLAS_package=OpenBLAS-0.3.7.tar.gz
#OPENBLAS的安装
install_openblas()
{
  export CC=$(which gcc)
  export CXX=$(which g++)
  export FC=$(which gfortran)
  tar -zxvf $PACKAGE_DIR/$OpenBLAS_package -C $OPENBLAS_DIR
  cd $OPENBLAS_DIR/OpenBLAS-0.3.7
  make
  make PREFIX=$OPENBLAS_DIR install
  export LD_LIBRARY_PATH=$OPENBLAS_DIR/lib:$LD_LIBRARY_PATH
}

#SCALAPACK的安装
install_scalapack()
{
  tar -zxvf $PACKAGE_DIR/$scalapack_package -C $SCALAPACK_DIR
  cd $SCALAPACK_DIR/scalapack-2.0.2
  cp SLmake.inc.example SLmake.inc
  sed -i '58,59d' SLmake.inc
  sed "58 iLAPACKLIB     = -L$OPENBLAS_DIR/lib -lopenblas" -i SLmake.inc
  sed "58 iBLASLIB       = -L$OPENBLAS_DIR/lib -lopenblas" -i SLmake.inc
  make
  cp libscalapack.a $SCALAPACK_DIR
  export LD_LIBRARY_PATH=$SCALAPACK_DIR:$LD_LIBRARY_PATH
}


install_all()
{
  env_check
  install_openblas
  install_scalapack
}

time_tag=$(echo $(date +%F-%H%M%S)_$(hostname))
install_all 2>&1 | tee -a $LOGS_DIR/install_QE_dependencies_aarch64_gcc_${time_tag}.log
end_time=$(date +%s)
spend_time=$(expr $end_time - $start_time)
set_color 33m "本次安装耗时：${spend_time}s"
